﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Runtime.ConstrainedExecution;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Threading.Tasks;
using System.Windows.Forms;
using AddinExpress.MSO;
using Microsoft.IdentityModel.Clients.ActiveDirectory;
using Microsoft.OData.ProxyExtensions;
using Microsoft.Office365.OutlookServices;
using Excel = Microsoft.Office.Interop.Excel;

namespace O365ExchangeSample
{
    /// <summary>
    ///   Add-in Express Add-in Module
    /// </summary>
    [GuidAttribute("5A9C1873-C833-4ABE-A837-A17A72507F40"), ProgId("O365ExchangeSample.AddinModule")]
    public partial class AddinModule : AddinExpress.MSO.ADXAddinModule
    {
        public AddinModule()
        {
            Application.EnableVisualStyles();
            InitializeComponent();
            // Please add any initialization code to the AddinInitialize event handler
        }

        #region Add-in Express automatic code

        // Required by Add-in Express - do not modify
        // the methods within this region

        public override System.ComponentModel.IContainer GetContainer()
        {
            if (components == null)
                components = new System.ComponentModel.Container();
            return components;
        }

        [ComRegisterFunctionAttribute]
        public static void AddinRegister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXRegister(t);
        }

        [ComUnregisterFunctionAttribute]
        public static void AddinUnregister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXUnregister(t);
        }

        public override void UninstallControls()
        {
            base.UninstallControls();
        }

        #endregion

        public static new AddinModule CurrentInstance
        {
            get
            {
                return AddinExpress.MSO.ADXAddinModule.CurrentInstance as AddinModule;
            }
        }

        public Excel._Application ExcelApp
        {
            get
            {
                return (HostApplication as Excel._Application);
            }
        }

        private string clientId = "bb4c578d-60ad-4406-873f-cdb05493100b";
        private string authority = "https://login.microsoftonline.com/common";
        private string redirectUri = "http://localhost/e62ac7d84f3adff953620a75c0a07592";
        private string resourceId = "https://outlook.office365.com/";
        private string accessToken = string.Empty;

        private int MESSAGE_CONTACTS = 0x0400 + 900;
        private int MESSAGE_EVENTS = 0x0400 + 901;
        private int MESSAGE_MAILS = 0x0400 + 902;

        private IPagedCollection contacts = null;
        private IPagedCollection events = null;
        private IPagedCollection mails = null;

        public void Authenticate()
        {
            try
            {
                var authContext = new AuthenticationContext(authority);
                var authResult1 = authContext.AcquireToken(resourceId, clientId, new Uri(redirectUri),
                    PromptBehavior.Auto);

                accessToken = authResult1.AccessToken;
            }
            catch
            {
                accessToken = string.Empty;
            }
        }

        private async void adxRibbonButtonGetContacts_OnClick(object sender, IRibbonControl control, bool pressed)
        {
            contacts = await GetContacts();
            this.SendMessage(MESSAGE_CONTACTS);
        }

        private async Task<IPagedCollection> GetContacts()
        {
            if (string.IsNullOrEmpty(accessToken))
                Authenticate();

            if (!string.IsNullOrEmpty(accessToken))
            {
                try
                {
                    var client = new OutlookServicesClient(new Uri("https://outlook.office365.com/api/v1.0"), async () => accessToken);

                    var contactResults = (IPagedCollection)await client.Me.Contacts
                        .OrderByDescending(m => m.DateTimeCreated)
                        .Take(10)
                        .Select(m => new { m.DisplayName, m.DateTimeCreated, m.CompanyName, m.JobTitle })
                        .ExecuteAsync();

                    return contactResults;
                }
                catch { }
            }
            return null;
        }

        private async void adxRibbonButtonGetEvents_OnClick(object sender, IRibbonControl control, bool pressed)
        {
            events = await GetEvents();
            this.SendMessage(MESSAGE_EVENTS);
        }

        private async Task<IPagedCollection> GetEvents()
        {
            if (string.IsNullOrEmpty(accessToken))
                Authenticate();

            if (!string.IsNullOrEmpty(accessToken))
            {
                try
                {
                    var client = new OutlookServicesClient(new Uri("https://outlook.office365.com/api/v1.0"), async () => accessToken);

                    var eventResults = (IPagedCollection)await client.Me.Events
                        .OrderByDescending(m => m.DateTimeCreated)
                        .Take(10)
                        .Select(m => new { m.Subject, m.DateTimeCreated, m.Start, m.End, m.Type })
                        .ExecuteAsync();

                    return eventResults;
                }
                catch { }
            }
            return null;
        }

        private async void adxRibbonButtonGetMail_OnClick(object sender, IRibbonControl control, bool pressed)
        {
            mails = await GetMails();
            this.SendMessage(MESSAGE_MAILS);
        }

        private async Task<IPagedCollection> GetMails()
        {
            if (string.IsNullOrEmpty(accessToken))
                Authenticate();

            if (!string.IsNullOrEmpty(accessToken))
            {
                try
                {
                    var client = new OutlookServicesClient(new Uri("https://outlook.office365.com/api/v1.0"), async () => accessToken);

                    var eventResults = (IPagedCollection)await client.Me.Messages
                        .OrderByDescending(m => m.DateTimeReceived)
                        .Take(10)
                        .Select(m => new { m.Subject, m.DateTimeReceived, m.From.EmailAddress.Name, m.HasAttachments })
                        .ExecuteAsync();

                    return eventResults;
                }
                catch { }
            }
            return null;
        }

        private void AddinModule_OnSendMessage(object sender, ADXSendMessageEventArgs e)
        {
            if (e.Message == MESSAGE_CONTACTS)
                HandleContacts();

            if (e.Message == MESSAGE_EVENTS)
                HandleEvents();

            if (e.Message == MESSAGE_MAILS)
                HandleMails();
        }

        private void HandleContacts()
        {
            Excel.Worksheet currSheet = null;
            Excel.Range displayNameRange = null;
            Excel.Range jobTitleRange = null;
            Excel.Range companyNameRange = null;

            try
            {
                if (contacts != null)
                {
                    currSheet = ExcelApp.ActiveSheet as Excel.Worksheet;
                    displayNameRange = currSheet.Range["A1"];
                    displayNameRange.Value = "Display Name";

                    jobTitleRange = currSheet.Range["B1"];
                    jobTitleRange.Value = "Job Title";

                    companyNameRange = currSheet.Range["C1"];
                    companyNameRange.Value = "Company Name";

                    int rowCount = 2;
                    foreach (dynamic contact in contacts.CurrentPage)
                    {
                        currSheet.Range["A" + rowCount].Value = contact.DisplayName;
                        currSheet.Range["B" + rowCount].Value = contact.JobTitle;
                        currSheet.Range["C" + rowCount].Value = contact.CompanyName;

                        rowCount++;
                    }
                }
            }
            finally
            {
                if (companyNameRange != null)
                    Marshal.ReleaseComObject(companyNameRange);
                if (jobTitleRange != null)
                    Marshal.ReleaseComObject(jobTitleRange);
                if (displayNameRange != null)
                    Marshal.ReleaseComObject(displayNameRange);
                if (currSheet != null)
                    Marshal.ReleaseComObject(currSheet);
            }
        }

        private void HandleEvents()
        {
            Excel.Worksheet currSheet = null;
            Excel.Range subjectRange = null;
            Excel.Range startRange = null;
            Excel.Range endRange = null;
            Excel.Range typeRange = null;

            try
            {
                if (events != null)
                {
                    currSheet = ExcelApp.ActiveSheet as Excel.Worksheet;
                    subjectRange = currSheet.Range["A1"];
                    subjectRange.Value = "Subject";

                    startRange = currSheet.Range["B1"];
                    startRange.Value = "Start";

                    endRange = currSheet.Range["C1"];
                    endRange.Value = "End";

                    typeRange = currSheet.Range["D1"];
                    typeRange.Value = "Type";

                    int rowCount = 2;
                    foreach (dynamic contact in events.CurrentPage)
                    {
                        currSheet.Range["A" + rowCount].Value = contact.Subject;
                        currSheet.Range["B" + rowCount].Value = contact.Start.ToString();
                        currSheet.Range["C" + rowCount].Value = contact.End.ToString();
                        currSheet.Range["D" + rowCount].Value = contact.Type.ToString();

                        rowCount++;
                    }
                }
            }
            finally
            {
                if (typeRange != null)
                    Marshal.ReleaseComObject(typeRange);
                if (endRange != null)
                    Marshal.ReleaseComObject(endRange);
                if (startRange != null)
                    Marshal.ReleaseComObject(startRange);
                if (subjectRange != null)
                    Marshal.ReleaseComObject(subjectRange);
                if (currSheet != null)
                    Marshal.ReleaseComObject(currSheet);
            }
        }

        private void HandleMails()
        {
            Excel.Worksheet currSheet = null;
            Excel.Range fromRange = null;
            Excel.Range subjectRange = null;
            Excel.Range dateReceivedRange = null;
            Excel.Range hasAttachmentRange = null;

            try
            {
                if (mails != null)
                {
                    currSheet = ExcelApp.ActiveSheet as Excel.Worksheet;
                    fromRange = currSheet.Range["A1"];
                    fromRange.Value = "From";

                    subjectRange = currSheet.Range["B1"];
                    subjectRange.Value = "Subject";

                    dateReceivedRange = currSheet.Range["C1"];
                    dateReceivedRange.Value = "Date Received";

                    hasAttachmentRange = currSheet.Range["D1"];
                    hasAttachmentRange.Value = "Has Attachments";

                    int rowCount = 2;
                    foreach (dynamic contact in mails.CurrentPage)
                    {
                        currSheet.Range["A" + rowCount].Value = contact.Name;
                        currSheet.Range["B" + rowCount].Value = contact.Subject;
                        currSheet.Range["C" + rowCount].Value = contact.DateTimeReceived.ToString();
                        currSheet.Range["D" + rowCount].Value = contact.HasAttachments.ToString();

                        rowCount++;
                    }
                }
            }
            finally
            {
                if (hasAttachmentRange != null)
                    Marshal.ReleaseComObject(hasAttachmentRange);
                if (dateReceivedRange != null)
                    Marshal.ReleaseComObject(dateReceivedRange);
                if (subjectRange != null)
                    Marshal.ReleaseComObject(subjectRange);
                if (fromRange != null)
                    Marshal.ReleaseComObject(fromRange);
                if (currSheet != null)
                    Marshal.ReleaseComObject(currSheet);
            }
        }

    }
}


// Install-Package Microsoft.IdentityModel.Clients.ActiveDirectory -Version 2.18.206251556 
// Install-Package Microsoft.Office365.OutlookServices

